package exceptionsArticle;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import java.net.Socket;
import java.net.UnknownHostException;

public class Chapter3
{
	public static void main(String[] args)
	{
		try	{throwArrayIndexOutOfBoundsException();}
		catch(ArrayIndexOutOfBoundsException aioobe) {System.err.println(aioobe);}
		
		try {throwClassCastException();}
		catch(ClassCastException cce) {System.err.println(cce);}
		
		try {throwIOException();}
		catch(IOException ioe)
		{
			System.err.println(ioe);
			System.err.println("Cause: " + ioe.getCause());
		}
		
		try {throwClassNotFoundException();}
		catch(ClassNotFoundException cnfe) {System.err.println(cnfe);}
		
		try {throwFileNotFoundException();}
		catch(FileNotFoundException fnfe) {System.err.println(fnfe);}
		
		try {throwNullPointerException();}
		catch(NullPointerException npe) {System.err.println(npe);}
		
		try {throwNumberFormatException();}
		catch(NumberFormatException nfe) {System.err.println(nfe);}
		
		try {throwUnknownHostException();}
		catch(UnknownHostException uhe) {System.err.println(uhe);}
	}
	
	/**
	 * This method will throw an ArrayIndexOutOfBoundsException.
	 * @throws ArrayIndexOutOfBoundsException
	 */
	public static void throwArrayIndexOutOfBoundsException() throws ArrayIndexOutOfBoundsException
	{
		int[] array = new int[5];
		for(int i=0; i<6; i++) // This will try to access array[5], which doesn't exist
		{
			array[i] = i;
		}
	}
	
	/**
	 * This method will throw a ClassCastException.
	 * @throws ClassCastException
	 */
	public static void throwClassCastException() throws ClassCastException
	{
		// Anything extends java.lang.Object directly or indirectly and is therefore an Object
		Object testClass1 = new Class1("1");
		Object testClass2 = (Class2) testClass1;
		System.out.println(testClass2.toString());
		
		// Now we've defined the cast ourselves and don't need to define the new Objects as Objects.
		Class1 testClass1_1 = new Class1("1");
		Class2 testClass2_1 = testClass1_1.toClass2();
		System.out.println(testClass2_1.toString());
		
		// If we do want to define them as Objects, it would work like this:
		Object testClass1_2 = new Class1("1");
		Object testClass2_2 = ((Class1) testClass1_2).toClass2();
		System.out.println(testClass2_2.toString());
	}
	
	/**
	 * This will throw a ClassNotFoundException, if Chapter4 wasn't compiled,
	 * when you try to run Chapter3. Most IDEs prevent this automaticly most of the time.
	 * @throws ClassNotFoundException
	 */
	public static void throwClassNotFoundException() throws ClassNotFoundException
	{
		Chapter4 chapter4 = new Chapter4();
		System.out.println(chapter4.toString());
	}
	
	/**
	 * This will throw a FileNotFoundException, if the File "hello.txt" doesn't exist.
	 * @throws FileNotFoundException
	 */
	public static void throwFileNotFoundException() throws FileNotFoundException
	{
		File file = new File("hello.txt");
		FileInputStream iStream = new FileInputStream(file);
		try{iStream.close();}catch(IOException ioe){}
	}
	
	/**
	 * This will throw an IOException, as the OutputStream can't be used
	 * after it was closed.
	 * @throws IOException
	 */
	public static void throwIOException() throws IOException
	{
		try
		{
			OutputStream oStream = new FileOutputStream(new File(""));
			oStream.close();
			oStream.write(0);
		}
		// The FileNotFoundException extends IOException - however to get a basic
		// IOException, we do this: 
		catch(FileNotFoundException fnfe)
		{
			IOException ioe = new IOException();//fnfe.getMessage());
			ioe.initCause(fnfe);
			throw ioe;
		}
	}
	
	/**
	 * A File for the throwNullPointerException method
	 */
	public static File file;
	/**
	 * Will throw a NullPointerException, as "file" was declared, but not defined.
	 * @throws NullPointerException
	 */
	public static void throwNullPointerException() throws NullPointerException
	{
		try
		{
			FileInputStream iStream = new FileInputStream(file);
			iStream.close();
		}
		catch(IOException ioe)
		{
			System.err.println(ioe);
		}
	}
	
	/**
	 * This will throw a NumberFormatException, as "hello" is no number.
	 * @throws NumberFormatException
	 */
	public static void throwNumberFormatException() throws NumberFormatException
	{
		int i = Integer.parseInt("hello");
		System.out.println(i);
	}
	
	public static void throwUnknownHostException() throws UnknownHostException
	{
		try
		{
			Socket socket = new Socket("nonexistentHost",0);
			socket.close();
		}
		catch(UnknownHostException uhe){throw uhe;}
		catch(IOException ioe){System.err.println("This Exception was not intended:\n\t" + ioe);}
	}
}
/**
 * Two additional classes needed for the ClassCastException example.
 */
class Class1
{
    String x;
    public Class1(String x)
    {
        this.x = x;
    }
    public Class2 toClass2()
    {
    	return new Class2(Integer.parseInt(x));
    }
}
 
class Class2
{
    int x;
    public Class2(int x)
    {
        this.x = x;
    }
}